/******************************************************************************
 * @file      protocol_mqtt_bcu.c
 * @brief     protocol_mqtt_bcu
 * @version   V1.0
 * @author    Gary Wangk
 * @copyright
 ******************************************************************************/

#include "protocol_mqtt_bcu.h"
#include "iso_check.h"
#include "soc_calculate.h"
#include "sop_calculate.h"
#include "version_manager.h"
#include "dido_manager.h"
#include "fault_manager.h"
#include "flash_manager.h"
#include "comm_manager.h"
#include "hv_adc_manager.h"
#include "eeprom_manager.h"
#include "signal_manager.h"
#include "statistic_manager.h"
#include "bmu_manager.h"
#include "protocol_comm.h"
#include "drv_qfc41d.h"

uint32_t bms_get_baseInfo(uint16_t input)
{
    uint32_t tmp;
    uint16_t bms_stu = 0;

    if(bms_get_run_status() == kRunStatus_Init)
    {
        bms_stu = 0;
    }
    else if(bms_get_run_status() == kRunStatus_Standby)
    {
        bms_stu |= 0x1;
    }
    else if(bms_get_run_status() == kRunStatus_Chg)
    {
        bms_stu |= 0x2;
    }
    else if(bms_get_run_status() == kRunStatus_Dis)
    {
        bms_stu |= 0x3;
    }
    if(bms_get_signal(kSignalIdx_ForbidChg) == kSignalStatus_High)
    {
        bms_stu |= 0x10;
    }
    if(bms_get_signal(kSignalIdx_ForbidDis) == kSignalStatus_High)
    {
        bms_stu |= 0x20;
    }

    switch(input)
    {
    case 1:
        tmp = bms_get_statistic_data(kStatisticData_TotalVolt);
        return tmp;
    case 2:
        tmp = bms_get_show_current();
        return tmp;
    case 3:
        tmp = bms_get_soc() / 100;
        return tmp;
    case 4:
        tmp = bms_get_soh() / 100;
        return tmp;
    case 5:
        tmp = bms_get_soc() / 100;
        return tmp;
    case 6:
        tmp = get_eeprom_data(kEep_RatedTotalVolt,kEepromDataType_Full);
        return tmp;
    case 7:
        tmp = get_eeprom_data(kEep_RatedCapacity,kEepromDataType_Full);
        return tmp;
    case 8:
        tmp = (uint32_t)bms_get_soc()*get_eeprom_data(kEep_RatedCapacity, kEepromDataType_Full)/10000;
        return tmp;
    case 9:
        tmp = get_eeprom_data(kEep_RatedCapacity, kEepromDataType_Full)*get_eeprom_data(kEep_RatedTotalVolt, kEepromDataType_Full)/10000;
        return tmp;
    case 10:
        tmp = (uint32_t)bms_get_soc() * get_eeprom_data(kEep_RatedCapacity, kEepromDataType_Full)/10000*get_eeprom_data(kEep_RatedTotalVolt, kEepromDataType_Full)/10000;
        return tmp;
    case 11:
        tmp = bms_get_bmu_num();   //
        return tmp;
    case 12:
        tmp = bms_get_bmu_online_num();
        return tmp;
    case 13:
        tmp = bms_get_cell_num();  //о
        return tmp;
    case 14:
        tmp = bms_get_statistic_data(kStatisticData_OnlineCellNum);
        return tmp;
    case 15:
        tmp = bms_get_temp_num();  //¸
        return tmp;
    case 16:
        tmp = bms_get_statistic_data(kStatisticData_OnlineTempNum);
        return tmp;
    case 17:
        tmp = bms_get_sop_data(kSopData_DisCur);
        return tmp;
    case 18:
        tmp = (uint32_t)bms_get_sop_data(kSopData_DisCur)*get_eeprom_data(kEep_RatedTotalVolt, kEepromDataType_Full)/10000;
        return tmp;
    case 19:
        tmp = bms_get_sop_data(kSopData_ChgCur);
        return tmp;
    case 20:
        tmp = (uint32_t)bms_get_sop_data(kSopData_ChgCur)*get_eeprom_data(kEep_RatedTotalVolt, kEepromDataType_Full)/10000;
        return tmp;
    case 21:
        tmp = bms_get_iso_data(kIsoData_PosRes);
        return tmp;
    case 22:
        tmp = bms_get_iso_data(kIsoData_NegRes);
        return tmp;
    case 23:
        tmp = bms_get_statistic_data(kStatisticData_AvgVolt);
        return tmp;
    case 24:
        tmp = bms_get_statistic_data(kStatisticData_VoltDiff);
        return tmp;
    case 25:
        tmp = bms_get_statistic_data(kStatisticData_MaxVolt);
        return tmp;
    case 26:
        tmp = bms_get_statistic_data(kStatisticData_MaxVoltBmuIdx) + 1;
        return tmp;
    case 27:
        tmp = bms_get_statistic_data(kStatisticData_MaxVoltIdx) + 1;
        return tmp;
    case 28:
        tmp = bms_get_statistic_data(kStatisticData_MinVolt);
        return tmp;
    case 29:
        tmp = bms_get_statistic_data(kStatisticData_MinVoltBmuIdx) + 1;
        return tmp;
    case 30:
        tmp = bms_get_statistic_data(kStatisticData_MinVoltIdx) + 1;
        return tmp;
    case 31:
        tmp = bms_get_statistic_data(kStatisticData_AvgTemp);
        return tmp;
    case 32:
        tmp = bms_get_statistic_data(kStatisticData_TempDiff);
        return tmp;
    case 33:
        tmp = bms_get_statistic_data(kStatisticData_MaxTemp);
        return tmp;
    case 34:
        tmp = bms_get_statistic_data(kStatisticData_MaxTempBmuIdx) + 1;
        return tmp;
    case 35:
        tmp = bms_get_statistic_data(kStatisticData_MaxTempIdx) + 1;
        return tmp;
    case 36:
        tmp = bms_get_statistic_data(kStatisticData_MinTemp);
        return tmp;
    case 37:
        tmp = bms_get_statistic_data(kStatisticData_MinTempBmuIdx) + 1;
        return tmp;
    case 38:
        tmp = bms_get_statistic_data(kStatisticData_MinTempIdx) + 1;
        return tmp;
    case 39:
        tmp = (uint16_t)bms_get_cumulate_data(kCumulateData_DayChgCap);
        return tmp;
    case 40:
        tmp = (uint16_t)bms_get_cumulate_data(kCumulateData_DayChgEnergy);
        return tmp;
    case 41:
        tmp = (uint16_t)bms_get_cumulate_data(kCumulateData_DayDisCap);
        return tmp;
    case 42:
        tmp = (uint16_t)bms_get_cumulate_data(kCumulateData_DayDisEnergy);
        return tmp;
    case 43:
        tmp = (uint16_t)bms_get_cumulate_data(kCumulateData_DayChgTime);
        return tmp;
    case 44:
        tmp = (uint16_t)bms_get_cumulate_data(kCumulateData_DayDisTime);
        return tmp;
    case 45:
        tmp = bms_get_cumulate_data(kCumulateData_AccChgCap);
        return tmp;
    case 46:
        tmp = bms_get_cumulate_data(kCumulateData_AccChgEnergy);
        return tmp;
    case 47:
        tmp = bms_get_cumulate_data(kCumulateData_AccDisCap);
        return tmp;
    case 48:
        tmp = bms_get_cumulate_data(kCumulateData_AccDisEnergy);
        return tmp;
    case 49:
        tmp = bms_get_cumulate_data(kCumulateData_AccChgTime);
        return tmp;
    case 50:
        tmp = bms_get_cumulate_data(kCumulateData_AccDisTime);
        return tmp;
    case 51:
        tmp = bms_stu;
        return tmp;
		case 52:
        tmp = bms_get_do_status_by_type(kDoType_PreChg);
        return tmp;
		case 53:
        tmp = bms_get_do_status_by_type(kDoType_Dis);
        return tmp;
		case 54:
        tmp = bms_get_do_status_by_type(kDoType_Chg);
        return tmp;
		case 55:
        tmp = 1;
        return tmp;
    default:
        return 0;
    }
}

uint32_t bms_get_fault_level_1(uint16_t input)
{
    uint32_t tmp = 0;

    switch(input)
    {
    case 1:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 2))  //һѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 2:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 3))  //һǷѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 3:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 4))  //һ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 4:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 5))  //һ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 5:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 6))  //һѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 6:
        if (bms_get_run_status() == kRunStatus_Chg)
        {
            if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 8))  //һ
            {
                tmp = 1;
            }
            else
            {
                tmp = 0;
            }
        }
        return tmp;
    case 7:
        if (bms_get_run_status() == kRunStatus_Dis)
        {
            if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 8))  //һŵ
            {
                tmp = 1;
            }
            else
            {
                tmp = 0;
            }
        }
        return tmp;
    case 8:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 14))  //һSOC
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 9:
        tmp = 0; //һSOC
        return tmp;
    case 10:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 15))  //һԵ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 11:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 0))  //һѹѹ--˹ѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 12:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 1))  //һѹǷѹ--Ƿѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
				return tmp;
    case 13:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(1, kFaultLevel_First), 12))  //1
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 14:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_First), 9))  //ѹ쳣
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 15:
        tmp = get_eeprom_data(kEep_TempNum, kEepromDataType_Full);
        return tmp;
    case 16:
        tmp = bms_get_statistic_data(kStatisticData_OnlineTempNum);
				return tmp;
        break;
    default:
        return 0;
    }
}

uint32_t bms_get_fault_level_2(uint16_t input)
{
    uint32_t tmp = 0;

    switch(input)
    {
    case 1:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 2))  //2ѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 2:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 3))  //2Ƿѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 3:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 4))  //2
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 4:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 5))  //2
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 5:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 6))  //2ѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 6:
        if (bms_get_run_status() == kRunStatus_Chg)
        {
            if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 8))  //2
            {
                tmp = 1;
            }
            else
            {
                tmp = 0;
            }
        }
        return tmp;
    case 7:
        if (bms_get_run_status() == kRunStatus_Dis)
        {
            if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 8))  //2ŵ
            {
                tmp = 1;
            }
            else
            {
                tmp = 0;
            }
        }
        return tmp;
    case 8:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 14))  //2SOC
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 9:
        tmp = 0; //һSOC
        return tmp;
    case 10:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 15))  //2Ե
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 11:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 0))  //2ѹѹ--˹ѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 12:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Second), 1))  //2ѹǷѹ--Ƿѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
				return tmp;
    case 13:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(1, kFaultLevel_Second), 12))  //2
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 14:
        tmp = bms_get_statistic_data(kStatisticData_OnlineCellNum);
        return tmp;
    case 15:
        tmp = get_eeprom_data(kEep_TempNum, kEepromDataType_Full);
        return tmp;
    case 16:
        tmp = bms_get_statistic_data(kStatisticData_OnlineTempNum);
        break;
    default:
        return 0;
    }
}

uint32_t bms_get_fault_level_3(uint16_t input)
{
    uint32_t tmp = 0;

    switch(input)
    {
    case 1:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 2))  //3ѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 2:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 3))  //3Ƿѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 3:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 4))  //3
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 4:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 5))  //3
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 5:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 6))  //3ѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 6:
        if (bms_get_run_status() == kRunStatus_Chg)
        {
            if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 8))  //3
            {
                tmp = 1;
            }
            else
            {
                tmp = 0;
            }
        }
        return tmp;
    case 7:
        if (bms_get_run_status() == kRunStatus_Dis)
        {
            if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 8))  //3ŵ
            {
                tmp = 1;
            }
            else
            {
                tmp = 0;
            }
        }
        return tmp;
    case 8:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 14))  //3SOC
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 9:
        tmp = 0; //һSOC
        return tmp;
    case 10:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 15))  //3Ե
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 11:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 0))  //3ѹѹ--˹ѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 12:  
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(0, kFaultLevel_Third), 1))  //3ѹǷѹ--Ƿѹ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
				return tmp;
    case 13:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(1, kFaultLevel_Third), 12))  //3
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
        return tmp;
    case 14:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(1, kFaultLevel_Third), 0))  //̵ճ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
				return tmp;
    case 15:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(2, kFaultLevel_Third), 9))  //޹
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
				return tmp;
    case 16:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(1, kFaultLevel_Third), 1))  //Ԥ
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
				return tmp;
    case 17:
        if (KIT_GET_BIT_32(bms_get_fault_single_bit(2, kFaultLevel_Third), 10))  //·
        {
            tmp = 1;
        }
        else
        {
            tmp = 0;
        }
				return tmp;
    default:
        return 0;
    }
}

//밴ģ
const devPointMap bms_points[] = {
		{"BCU_1", 0, 55, bms_get_baseInfo}, // 豸
    {"BCU_2", 0, 0, bms_get_total_alarm}, // ܸ澯
    {"BCU_3", 0, 0, bms_get_total_fault}, // ܹ
    {"BCU_4", 0, 1, bms_get_fault_level_1}, // ѹ澯һ
    {"BCU_5", 0, 2, bms_get_fault_level_1}, // Ƿѹ澯һ
    {"BCU_6", 0, 3, bms_get_fault_level_1}, // ¸澯һ
    {"BCU_7", 0, 4, bms_get_fault_level_1}, // ¸澯һ
    {"BCU_8", 0, 5, bms_get_fault_level_1}, // ѹ澯һ
    {"BCU_9", 0, 6, bms_get_fault_level_1}, // 澯һ
    {"BCU_10", 0, 7, bms_get_fault_level_1}, // ŵ澯һ
    {"BCU_11", 0, 8, bms_get_fault_level_1}, // SOC͸澯һ
    {"BCU_12", 0, 9, bms_get_fault_level_1}, // SOC澯һ
    {"BCU_13", 0, 10, bms_get_fault_level_1}, // Ե͸澯һ

    {"BCU_14", 0, 1, bms_get_fault_level_2}, // ѹ澯
    {"BCU_15", 0, 2, bms_get_fault_level_2}, // Ƿѹ澯
    {"BCU_16", 0, 3, bms_get_fault_level_2}, // ¸澯
    {"BCU_17", 0, 4, bms_get_fault_level_2}, // ¸澯
    {"BCU_18", 0, 5, bms_get_fault_level_2}, // ѹ澯
    {"BCU_19", 0, 6, bms_get_fault_level_2}, // 澯
    {"BCU_20", 0, 7, bms_get_fault_level_2}, // ŵ澯
    {"BCU_21", 0, 8, bms_get_fault_level_2}, // SOC͸澯
    {"BCU_22", 0, 9, bms_get_fault_level_2}, // SOC澯
    {"BCU_23", 0, 10, bms_get_fault_level_2}, // Ե͸澯

    {"BCU_24", 0, 1, bms_get_fault_level_3}, // ѹ澯
    {"BCU_25", 0, 2, bms_get_fault_level_3}, // Ƿѹ澯
    {"BCU_26", 0, 3, bms_get_fault_level_3}, // ¸澯
    {"BCU_27", 0, 4, bms_get_fault_level_3}, // ¸澯
    {"BCU_28", 0, 5, bms_get_fault_level_3}, // ѹ澯
    {"BCU_29", 0, 6, bms_get_fault_level_3}, // 澯
    {"BCU_30", 0, 7, bms_get_fault_level_3}, // ŵ澯
    {"BCU_31", 0, 8, bms_get_fault_level_3}, // SOC͸澯
    {"BCU_32", 0, 9, bms_get_fault_level_3}, // SOC澯
    {"BCU_33", 0, 10, bms_get_fault_level_3}, // Ե͸澯

    {"BCU_34", 0, 0, NULL}, // о¶ȼ޸澯
    {"BCU_35", 0, 0, NULL}, // оѹ޸澯
    {"BCU_36", 0, 0, NULL}, // ؼ价1澯
    {"BCU_37", 0, 0, NULL}, // ؼ价2澯
    {"BCU_38", 0, 0, NULL}, // ؼ价3澯
    {"BCU_39", 0, 0, NULL}, // ؼ1澯
    {"BCU_40", 0, 0, NULL}, // ؼ2澯
		
    {"BCU_41", 1, 0, NULL}, // ؼ3澯
    {"BCU_42", 1, 11, bms_get_fault_level_1}, // ˹ѹ1澯
    {"BCU_43", 1, 11, bms_get_fault_level_2}, // ˹ѹ2澯
    {"BCU_44", 1, 11, bms_get_fault_level_3}, // ˹ѹ3澯
    {"BCU_45", 1, 12, bms_get_fault_level_1}, // Ƿѹ1澯
    {"BCU_46", 1, 12, bms_get_fault_level_2}, // Ƿѹ2澯
    {"BCU_47", 1, 12, bms_get_fault_level_3}, // Ƿѹ3澯
    {"BCU_48", 1, 13, bms_get_fault_level_1}, // 1澯
    {"BCU_49", 1, 13, bms_get_fault_level_2}, // 2澯
    {"BCU_50", 1, 13, bms_get_fault_level_3}, // 3澯

    {"BCU_51", 1, 0, NULL}, // AFE¸쳣
    {"BCU_52", 1, 0, NULL}, // AFEѹ쳣
    {"BCU_53", 1, 0, NULL}, // شͨŸ澯
    {"BCU_54", 1, 0, NULL}, // ͨѶ澯
		
    {"BCU_55", 1, 0, NULL}, // ˿
    {"BCU_56", 1, 0, NULL}, // ·
    {"BCU_57", 1, 0, NULL}, // յ
    {"BCU_58", 1, 0, NULL}, // 豸
    {"BCU_59", 1, 0, NULL}, // 
    {"BCU_60", 1, 0, NULL}, // 
    {"BCU_61", 1, 0, NULL}, // AFE
    {"BCU_62", 1, 14, bms_get_fault_level_1}, // ѹ쳣
    {"BCU_63", 1, 16, bms_get_fault_level_3}, // Ԥ澯
    {"BCU_64", 1, 17, bms_get_fault_level_3}, // ·
		
    {"BCU_65", 1, 14, bms_get_fault_level_3}, // ̵ճ澯
    {"BCU_66", 1, 15, bms_get_fault_level_3}, // ؼ޹
		
		{"BCU_67", 1, 52, bms_get_baseInfo}, // Ԥ̵
    {"BCU_68", 1, 53, bms_get_baseInfo}, // ̵
    {"BCU_69", 1, 54, bms_get_baseInfo}, // ̵ܸ
		
		{"BCU_101", 1, 1, bms_get_baseInfo}, // ѹ
		{"BCU_102", 1, 2, bms_get_baseInfo}, // ܵ
		{"BCU_103", 1, 3, bms_get_baseInfo}, // SOC
		{"BCU_104", 1, 4, bms_get_baseInfo}, // SOH
		{"BCU_105", 1, 5, bms_get_baseInfo}, // SOE
		{"BCU_106", 1, 6, bms_get_baseInfo}, // ѹ
		{"BCU_107", 1, 7, bms_get_baseInfo}, // 
		{"BCU_108", 1, 8, bms_get_baseInfo}, // ʣ
		{"BCU_109", 1, 9, bms_get_baseInfo}, // 
		{"BCU_110", 1, 10, bms_get_baseInfo}, // ʣ
		{"BCU_111", 1, 11, bms_get_baseInfo}, // ӻ(BMU)
		
		{"BCU_112", 2, 12, bms_get_baseInfo}, // ߴӻ(BMU)
		{"BCU_113", 2, 13, bms_get_baseInfo}, // 
		{"BCU_114", 2, 14, bms_get_baseInfo}, // ߵ
		{"BCU_115", 2, 15, bms_get_baseInfo}, // ¸
		{"BCU_116", 2, 16, bms_get_baseInfo}, // ¸
		{"BCU_117", 2, 17, bms_get_baseInfo}, // ŵ
		{"BCU_118", 2, 18, bms_get_baseInfo}, // ŵ繦
		{"BCU_119", 2, 19, bms_get_baseInfo}, // 
		{"BCU_120", 2, 20, bms_get_baseInfo}, // 繦
		{"BCU_121", 2, 21, bms_get_baseInfo}, // Եֵ
		{"BCU_122", 2, 22, bms_get_baseInfo}, // Եֵ
		{"BCU_123", 2, 23, bms_get_baseInfo}, // ƽѹ
		{"BCU_124", 2, 24, bms_get_baseInfo}, // ѹ
		{"BCU_125", 2, 25, bms_get_baseInfo}, // ߵѹ
		{"BCU_126", 2, 26, bms_get_baseInfo}, // ߵѹӻ
		{"BCU_127", 2, 27, bms_get_baseInfo}, // ߵѹ
		{"BCU_128", 2, 28, bms_get_baseInfo}, // ͵ѹ
		{"BCU_129", 2, 29, bms_get_baseInfo}, // ͵ѹӻ
		{"BCU_130", 2, 30, bms_get_baseInfo}, // ͵ѹ
		{"BCU_131", 2, 31, bms_get_baseInfo}, // ƽ¶
		{"BCU_132", 2, 32, bms_get_baseInfo}, // ²
		{"BCU_133", 2, 33, bms_get_baseInfo}, // ߵ¶
		{"BCU_134", 2, 34, bms_get_baseInfo}, // ߵ¶ȴӻ
		{"BCU_135", 2, 35, bms_get_baseInfo}, // ߵ¶ȱ
		{"BCU_136", 2, 36, bms_get_baseInfo}, // ͵¶
		{"BCU_137", 2, 37, bms_get_baseInfo}, // ͵¶ȴӻ
		{"BCU_138", 2, 38, bms_get_baseInfo}, // ͵¶ȱ
		{"BCU_139", 2, 39, bms_get_baseInfo}, // ճ
		{"BCU_140", 2, 40, bms_get_baseInfo}, // ճ
		{"BCU_141", 2, 41, bms_get_baseInfo}, // շŵ
		{"BCU_142", 2, 42, bms_get_baseInfo}, // շŵ
		{"BCU_143", 2, 43, bms_get_baseInfo}, // ճʱ
		{"BCU_144", 2, 44, bms_get_baseInfo}, // շŵʱ
		{"BCU_145", 2, 45, bms_get_baseInfo}, // ۼƳ
		{"BCU_146", 2, 46, bms_get_baseInfo}, // ۼƳ
		{"BCU_147", 2, 47, bms_get_baseInfo}, // ۼƷŵ
		{"BCU_148", 2, 48, bms_get_baseInfo}, // ۼƷŵ
		{"BCU_149", 2, 49, bms_get_baseInfo}, // ۼƳʱ
		{"BCU_150", 2, 50, bms_get_baseInfo}, // ۼƷŵʱ
		{"BCU_151", 2, 51, bms_get_baseInfo}, // BCU״̬
		
//    {"BCU_67", 1, 1, bms_get_baseInfo}, // ѹ
//    {"BCU_68", 1, 2, bms_get_baseInfo}, // ܵ
//    {"BCU_69", 1, 3, bms_get_baseInfo}, // SOC
//    {"BCU_70", 1, 4, bms_get_baseInfo}, // SOH
//    {"BCU_71", 1, 5, bms_get_baseInfo}, // SOE
//    {"BCU_72", 1, 6, bms_get_baseInfo}, // ѹ
//    {"BCU_73", 1, 7, bms_get_baseInfo}, // 
//    {"BCU_74", 1, 8, bms_get_baseInfo}, // ʣ
//    {"BCU_75", 1, 9, bms_get_baseInfo}, // 
//    {"BCU_76", 1, 10, bms_get_baseInfo}, // ʣ
//    {"BCU_77", 1, 11, bms_get_baseInfo}, // ӻ(BMU)
//    {"BCU_78", 1, 12, bms_get_baseInfo}, // ߴӻ(BMU)
//    {"BCU_79", 1, 13, bms_get_baseInfo}, // 
//    {"BCU_80", 1, 14, bms_get_baseInfo}, // ߵ
//    {"BCU_81", 1, 15, bms_get_baseInfo}, // ¸

//    {"BCU_82", 2, 16, bms_get_baseInfo}, // ¸
//    {"BCU_83", 2, 17, bms_get_baseInfo}, // ŵ
//    {"BCU_84", 2, 18, bms_get_baseInfo}, // ŵ繦
//    {"BCU_85", 2, 19, bms_get_baseInfo}, // 
//    {"BCU_86", 2, 20, bms_get_baseInfo}, // 繦
//    {"BCU_87", 2, 21, bms_get_baseInfo}, // Եֵ
//    {"BCU_88", 2, 22, bms_get_baseInfo}, // Եֵ
//    {"BCU_89", 2, 23, bms_get_baseInfo}, // ƽѹ
//    {"BCU_90", 2, 24, bms_get_baseInfo}, // ѹ
//    {"BCU_91", 2, 25, bms_get_baseInfo}, // ߵѹ
//    {"BCU_92", 2, 26, bms_get_baseInfo}, // ߵѹӻ
//    {"BCU_93", 2, 27, bms_get_baseInfo}, // ߵѹ
//    {"BCU_94", 2, 28, bms_get_baseInfo}, // ͵ѹ
//    {"BCU_95", 2, 29, bms_get_baseInfo}, // ͵ѹӻ
//    {"BCU_96", 2, 30, bms_get_baseInfo}, // ͵ѹ
//    {"BCU_97", 2, 31, bms_get_baseInfo}, // ƽ¶
//    {"BCU_98", 2, 32, bms_get_baseInfo}, // ²
//    {"BCU_99", 2, 33, bms_get_baseInfo}, // ߵ¶
//    {"BCU_100", 2, 34, bms_get_baseInfo}, // ߵ¶ȴӻ
//    {"BCU_101", 2, 35, bms_get_baseInfo}, // ߵ¶ȱ
//    {"BCU_102", 2, 36, bms_get_baseInfo}, // ͵¶
//    {"BCU_103", 2, 37, bms_get_baseInfo}, // ͵¶ȴӻ
//    {"BCU_104", 2, 38, bms_get_baseInfo}, // ͵¶ȱ
//    {"BCU_105", 2, 39, bms_get_baseInfo}, // ճ
//    {"BCU_106", 2, 40, bms_get_baseInfo}, // ճ
//    {"BCU_107", 2, 41, bms_get_baseInfo}, // շŵ
//    {"BCU_108", 2, 42, bms_get_baseInfo}, // շŵ
//    {"BCU_109", 2, 43, bms_get_baseInfo}, // ճʱ
//    {"BCU_110", 2, 44, bms_get_baseInfo}, // շŵʱ
//    {"BCU_111", 2, 45, bms_get_baseInfo}, // ۼƳ
//    {"BCU_112", 2, 46, bms_get_baseInfo}, // ۼƳ
//    {"BCU_113", 2, 47, bms_get_baseInfo}, // ۼƷŵ
//    {"BCU_114", 2, 48, bms_get_baseInfo}, // ۼƷŵ
//    {"BCU_115", 2, 49, bms_get_baseInfo}, // ۼƳʱ
//    {"BCU_116", 2, 50, bms_get_baseInfo}, // ۼƷŵʱ
//    {"BCU_117", 2, 51, bms_get_baseInfo}, // BCU״̬
};

const int bms_point_count = sizeof(bms_points) / sizeof(bms_points[0]);

/*
    const char* key;                		// JSON еļ "BCU_"治֣ݺͨBCU_pointId
    uint16_t pointId;						// ڼ¼ʼĲţbcuģ
	uint16_t startCellIndex;				// ʼĵѹţΣ
	uint16_t endCellIndex;					// ĵѹţΣ
    uint32_t (*get_val)(uint16_t);          // ӦֵĻȡ
*/
const devCellPointMap bms_volt_points[] =
{
    {"BCU_", 191, 0, 39, bms_get_cell_volt}, // 1~40ŵصѹ
    {"BCU_", 191, 40, 79, bms_get_cell_volt}, // 41~80ŵصѹ
    {"BCU_", 191, 80, 119, bms_get_cell_volt}, // 81~120ŵصѹ
    {"BCU_", 191, 120, 159, bms_get_cell_volt}, // 121~160ŵصѹ
    {"BCU_", 191, 160, 199, bms_get_cell_volt}, // 161~200ŵصѹ
    {"BCU_", 191, 200, 239, bms_get_cell_volt}, // 201~240ŵصѹ
    {"BCU_", 191, 240, 279, bms_get_cell_volt}, // 241~280ŵصѹ
    {"BCU_", 191, 280, 319, bms_get_cell_volt}, // 281~320ŵصѹ
    {"BCU_", 191, 320, 359, bms_get_cell_volt}, // 321~360ŵصѹ
    {"BCU_", 191, 360, 399, bms_get_cell_volt}, // 361~400ŵصѹ
};

const devCellPointMap bms_temp_points[] =
{
    {"BCU_", 611, 0, 39, bms_get_cell_temp}, // 1~40ŵ¶
    {"BCU_", 611, 40, 79, bms_get_cell_temp}, // 41~80ŵ¶
    {"BCU_", 611, 80, 99, bms_get_cell_temp}, // 81~100ŵ¶
};

const int bms_volt_count = sizeof(bms_volt_points) / sizeof(devCellPointMap);
const int bms_temp_count = sizeof(bms_temp_points) / sizeof(devCellPointMap);


void protocol_build_json(uint16_t groupId, uint32_t time) 
{
		int val = 0;
	
    cJSON* root = cJSON_CreateObject();

    // ʱ
    cJSON_AddNumberToObject(root, "timeStamp", time);

    // devData 飨һ豸
    cJSON* devDataArr = cJSON_CreateArray();
    cJSON_AddItemToObject(root, "devData", devDataArr);

    // 豸
    cJSON* deviceObj = cJSON_CreateObject();
    cJSON_AddItemToArray(devDataArr, deviceObj);

    // ӹؼֶ
    cJSON_AddStringToObject(deviceObj, "devType", "4");
    cJSON_AddStringToObject(deviceObj, "devName", "BCU");
    cJSON_AddStringToObject(deviceObj, "devId", "001");
    cJSON_AddStringToObject(deviceObj, "sn", "SN11223344");
    cJSON_AddNumberToObject(deviceObj, "currentPack", groupId);	
    cJSON_AddNumberToObject(deviceObj, "totalPack", 16);

		cJSON* dataObj = cJSON_CreateObject();
    cJSON_AddItemToObject(deviceObj, "data", dataObj);

		for (int i = 0; i < bms_point_count; ++i)
    {
        const devPointMap* point = &bms_points[i];
        if(point->groupId == groupId)
        {
            if (point->get_val != NULL)
            {
                val = point->get_val(point->input);
            }
            else
            {
                val = 0; // Ĭֵ
            }
						cJSON_AddNumberToObject(dataObj, point->key, val);
//					cJSON_AddNumberToObject(dataObj, point->key, 0);
        }
			//cJSON_AddNumberToObject(deviceObj, point->key, val); @wangk  鹦ʧЧ
    }


    char* json_str = cJSON_PrintUnformatted(root);
    if (json_str) 
    {
        drv_mqtt_publish_no_respose(json_str, strlen(json_str));
				bsp_task_delay_ms(50);
        cJSON_free(json_str);
    } 

    cJSON_Delete(root);
}

void protocol_build_volt_json(uint8_t i, uint32_t time)
{
	const devCellPointMap* item = &bms_volt_points[i];
    int val = 0;
    char key[10] = {0};
		uint8_t groupId = i + 3;
		
    cJSON* root = cJSON_CreateObject();

    cJSON_AddNumberToObject(root, "timeStamp", time);

    // devData 
    cJSON* devDataArr = cJSON_CreateArray();
    cJSON_AddItemToObject(root, "devData", devDataArr);

    // ÿһ豸
    cJSON* deviceObj = cJSON_CreateObject();
    cJSON_AddItemToArray(devDataArr, deviceObj);

    cJSON_AddStringToObject(deviceObj, "devType", "4");
    cJSON_AddStringToObject(deviceObj, "devName", "BCU");
    cJSON_AddStringToObject(deviceObj, "devId", "001");
    cJSON_AddStringToObject(deviceObj, "sn", "SN11223344");
		cJSON_AddNumberToObject(deviceObj, "currentPack", groupId);  // ǰ
    cJSON_AddNumberToObject(deviceObj, "totalPack", 16);       // ܰ

    //  data 
    cJSON* dataObj = cJSON_CreateObject();
    cJSON_AddItemToObject(deviceObj, "data", dataObj);


    for (int i = item->startCellIndex; i <= item->endCellIndex; ++i)
    {
				sprintf(key,"%s%d",item->key,item->pointId + i);
        if (item->get_val != NULL)
        {
            val = item->get_val(i);
        }
        else
        {
            val = 0; // Ĭֵ
        }
        cJSON_AddNumberToObject(dataObj, key, val);
//			cJSON_AddNumberToObject(dataObj, key, 0);
    }

    // תΪ JSON ַ
    char* json_str = cJSON_PrintUnformatted(root);
    if (json_str)
    {
        drv_mqtt_publish_no_respose(json_str, strlen(json_str));
				bsp_task_delay_ms(50);
        cJSON_free(json_str);
    }

    cJSON_Delete(root);
}

void protocol_build_temp_json(uint8_t i, uint32_t time)
{
		const devCellPointMap* item = &bms_temp_points[i];
    int val = 0;
    char key[10] = {0};
		uint8_t groupId = i + 13;
		
    cJSON* root = cJSON_CreateObject();

    cJSON_AddNumberToObject(root, "timeStamp", time);

    // devData 
    cJSON* devDataArr = cJSON_CreateArray();
    cJSON_AddItemToObject(root, "devData", devDataArr);

    // ÿһ豸
    cJSON* deviceObj = cJSON_CreateObject();
    cJSON_AddItemToArray(devDataArr, deviceObj);

    cJSON_AddStringToObject(deviceObj, "devType", "4");
    cJSON_AddStringToObject(deviceObj, "devName", "BCU");
    cJSON_AddStringToObject(deviceObj, "devId", "001");
    cJSON_AddStringToObject(deviceObj, "sn", "SN11223344");
		cJSON_AddNumberToObject(deviceObj, "currentPack", groupId);  // ǰ
    cJSON_AddNumberToObject(deviceObj, "totalPack", 16);       // ܰ

    //  data 
    cJSON* dataObj = cJSON_CreateObject();
    cJSON_AddItemToObject(deviceObj, "data", dataObj);


    for (int i = item->startCellIndex; i <= item->endCellIndex; ++i)
    {
				sprintf(key,"%s%d",item->key,item->pointId + i);
        if (item->get_val != NULL)
        {
            val = item->get_val(i);
        }
        else
        {
            val = 0; // Ĭֵ
        }
        cJSON_AddNumberToObject(dataObj, key, val);
//			cJSON_AddNumberToObject(dataObj, key, 0);
    }

    // תΪ JSON ַ
    char* json_str = cJSON_PrintUnformatted(root);
    if (json_str)
    {
        drv_mqtt_publish_no_respose(json_str, strlen(json_str));
				bsp_task_delay_ms(50);
        cJSON_free(json_str);
    }

    cJSON_Delete(root);
}

void publish_all_bms_groups()
{
    uint8_t max_group = 0;
		uint8_t i = 0;
		uint32_t timeTemp = 0;
	
		timeTemp = drv_rtc_get_tick();
    for (i = 0; i < bms_point_count; i++)
    {
        if (bms_points[i].groupId > max_group)
        {
            max_group = bms_points[i].groupId;
        }
    }

    for (uint8_t g = 0; g <= max_group; g++)
    {
        protocol_build_json(g, timeTemp);
    }
	
    for (i = 0; i < bms_volt_count; i++)
    {
        protocol_build_volt_json(i, timeTemp);
    }
	
    for (i = 0; i < bms_temp_count; i++)
    {
        protocol_build_temp_json(i, timeTemp);
    }
	
}

void mqtt_publish_bms_data(uint32_t basetime)
{
    static uint32_t mqtt_cycle_tick  = 0;
		//static uint16_t mqtt_init_count = 0;
		static uint8_t mqtt_init_flag = 1;
    static uint8_t groupMax = 0;

    mqtt_cycle_tick += basetime;
		
    if(mqtt_cycle_tick > 1000)
    {
//				if (!drv_at_send_cmd("AT+QSTAST", "+QSTAST:STATION_UP", WIFI_TIMEOUT))
//				{
//						mqtt_init_flag = 1;
//				}
//				
//				if (mqtt_init_flag == 1)
//				{	
//						if (drv_mqtt_connect() == 0)
//						{
//								mqtt_init_flag = 0;
//						}
//				}
				// ڼǷmqttɹ
				if (!drv_at_send_cmd("AT+QMTCONN?", "3\0", WIFI_TIMEOUT)) 
				{
						drv_mqtt_connect();
				}
//				else  //@wangk add 5-16,עͿ֤ifˣֻмûӵmqttʱ򣬲ŻִӺԴ
//				{
//						drv_mqtt_publish_no_respose("1", 1);
//				}

				// ڼǷ
				if (!drv_at_send_cmd("AT+QBLESTAT", "ADVERTISING\0", WIFI_TIMEOUT)) 
				{
						if (!drv_at_send_cmd("AT+QBLESTAT", "CONNECTED\0", WIFI_TIMEOUT)) 
						{
								drv_ble_init();
						}
				}
//				else   //@wangk add 5-20,עͿ֤ifˣСʻʧ
//				{
//						drv_mqtt_publish_no_respose("2", 1);
//				}

			
				
//				drv_wireless_send_string("AT+QMTPUB=0,1,2,0,\"BCU01\",17,\"Hello from FC41D!\"");
        mqtt_cycle_tick = 0;
        publish_all_bms_groups();

    }

}

